// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/driver/io.c $
// $Rev: 44983 $
// $Date: 2019-05-23 17:38:50 -0500 (Thu, 23 May 2019) $

// 16AO4MF: Device Driver: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dev_io_close
*
*	Purpose:
*
*		Cleanup the I/O stuff for the channel as it is being closed.
*
*	Arguments:
*
*		chan	The channel data structure.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_close(chan_data_t* chan)
{
	int	i;

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (chan->io.io_streams[i])
		{
			if (chan->io.io_streams[i]->dev_io_close)
			{
				(chan->io.io_streams[i]->dev_io_close)(chan, chan->io.io_streams[i]);
			}
		}
	}
}



/******************************************************************************
*
*	Function:	dev_io_create
*
*	Purpose:
*
*		Perform I/O based initialization as the driver is being loaded.
*
*	Arguments:
*
*		chan	The data for the channel of interest.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error code.
*
******************************************************************************/

int dev_io_create(chan_data_t* chan)
{
	int	ret;

	chan->io.io_streams[0]	= &chan->io.tx;

	ret	= dev_write_create(chan, &chan->io.tx);

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_io_destroy
*
*	Purpose:
*
*		Perform I/O based cleanup as the driver is being unloaded.
*
*	Arguments:
*
*		chan	The data for the channel of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_destroy(chan_data_t* chan)
{
	int	i;

	dev_io_close(chan);	// Just in case.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (chan->io.io_streams[i])
		{
			gsc_io_destroy(chan, chan->io.io_streams[i]);
		}
	}
}



/******************************************************************************
*
*	Function:	dev_io_open
*
*	Purpose:
*
*		Perform initialization work for a device that is being opened.
*
*	Arguments.
*
*		chan	The channel data structure.
*
*	Returned:
*
*		0	All went well.
*		< 0	An appropriate error code.
*
******************************************************************************/

int dev_io_open(chan_data_t* chan)
{
	int	i;

	dev_io_close(chan);	// Just in case.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (chan->io.io_streams[i])
		{
			if (chan->io.io_streams[i]->dev_io_open)
			{
				(chan->io.io_streams[i]->dev_io_open)(chan, chan->io.io_streams[i]);
			}
		}
	}

	return(0);
}



/******************************************************************************
*
*	Function:	dev_io_write_select
*
*	Purpose:
*
*		Select the dev_io_t structure for the referenced I/O write stream.
*
*	Arguments:
*
*		chan	The data for the channel of interest.
*
*		count	The number of bytes to transfer, with the I/O selector encoded
*				in the 0xF0000000 nibble.
*
*	Returned:
*
*		0		All went well.
*		< 0		The code for the error seen.
*
******************************************************************************/

dev_io_t* dev_io_write_select(chan_data_t* chan, size_t select)
{
	dev_io_t*	io;

	// We have only one output stream.
	io	= &chan->io.tx;
	return(io);
}



