// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/driver/main.h $
// $Rev: 52837 $
// $Date: 2023-04-11 12:37:25 -0500 (Tue, 11 Apr 2023) $

// 16AO4MF: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "16ao4mf.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)->dev
#define	GSC_ALT_STRUCT_GET(i,d)				gsc_alt_struct_get((i),(d))
#define	GSC_ALT_STRUCT_T					chan_data_t
#define	GSC_DEVS_PER_BOARD					4

typedef struct _chan_data_t					chan_data_t;
typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"16AO4MF"	// Upper case form of the below.
#define	DEV_NAME							"16ao4mf"	// MUST AGREE WITH AO4MF_BASE_NAME

#define	DEV_VERSION							"3.7"	// FOR DEVICE SPECIFIC CODE ONLY!
// 3.7	Updated to support the 6.x series kernel.
//		Changed register implementation - MUST REBUILD APPLICATION.
//		Made the initialize IOCTL identical for all channels.
//		Updated PLX code segregation.
// 3.6	Added support for the 5.x kernel series.
//		Removed compiler warnings for Fedora 31.
//		Updated for porting to Windows.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		LINTed sources.
// 3.5	Removed duplicate macros.
//		Corrected some comments.
//		White space cleanup.
//		Relocated some code.
//		Updated initial file content for consistency.
// 3.4	Updated per changes to common I/O releated code.
// 3.3	Changed "printk" calls to "printf" for cross platform compatibility.
// 3.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Updated initial file content for consistency.
//		Enabled metrics support.
//		Open now succeeds even if initialization fails.
// 3.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Minor correction to initialization code.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
// 3.0	Updated per changes to the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		LINTed the sources.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Migrated dev_irq_open()/close() to gsc_irq_open()/close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		Fixed a bug in the shutdown code.
// 2.0	Driver overhauled and rewritten.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				0
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AO4MF_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AO4MF_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AO4MF_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of channel's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of channel's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _chan_data_t
{
	os_sem_t			sem;		// control process access
	os_data_t			os;			// OS specific data for each device/channel
	struct _dev_data_t*	dev;
	int					index;		// 0-3
	int					dev_index;	// 0-3, 4-7, ...
	int					users;		// Number of currently active open() requests.
	int					share;		// Were we opened in shared mode?

	struct
	{					// This is for streaming I/O
		dev_io_t		tx;
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	gsc_wait_node_t*	wait_list;

	struct
	{
		s32				reg_diopcr;		// Digital I/O Port Control Register: Supported?

		s32				sync_clk_out;	// Are SYNC OUT and CLK OUT supported?

	} cache;

	struct
	{
		VADDR_T			gsc_accr_32;	// Adjustable Clock Control Register
		VADDR_T			gsc_bcr_32;		// Board Control Register
		VADDR_T			gsc_bor_32;		// Buffer Operations Register
		VADDR_T			gsc_diopcr_32;	// Digital I/O Port Control Register
		VADDR_T			gsc_icscr_32;	// Internal Clock Source Control Register
		VADDR_T			gsc_odbr_32;	// Output Data Buffer Register
		VADDR_T			gsc_srcr_32;	// Sample Rate Control Register
	} vaddr;

};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	char				model_buf[32];	// i.e. "16AO4MF"

	gsc_bar_t			bar;			// device register mappings
	chan_data_t			channel[GSC_DEVS_PER_BOARD];
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.

	struct
	{
		s32				auto_cal_ms;	// Max Auto Cal period in ms.

		s32				channel_qty;	// 4

		s32				fref_adj;		// The adjustable reference frequency.
		s32				fref_master;	// The master reference frequency.
		s32				fsamp_max;		// Maximum sample rate.
		s32				fsamp_min;		// Minimum sample rate.

		s32				init_ms;		// Max Init period in ms.

		s32				nclk_max;		// Maximum Nclk value.
		s32				nclk_min;		// Minimum Nclk value.
		s32				nrate_max;		// Maximum Nrate value.
		s32				nrate_min;		// Minimum Nrate value.

	} cache;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register
	} vaddr;

};



// prototypes *****************************************************************

void		dev_io_close(GSC_ALT_STRUCT_T* alt);
int			dev_io_create(chan_data_t* chan);
void		dev_io_destroy(chan_data_t* chan);
int			dev_io_open(GSC_ALT_STRUCT_T* alt);
dev_io_t*	dev_io_write_select(GSC_ALT_STRUCT_T* alt, size_t bytes);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_write_create(chan_data_t* chan, dev_io_t* io);

int			initialize_ioctl(chan_data_t* chan, s32* arg);
void		irq_reg_mod(chan_data_t* chan, VADDR_T va, u32 value, u32 mask);



#endif
