// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/aout/perform.c $
// $Rev: 52823 $
// $Date: 2023-04-11 12:23:08 -0500 (Tue, 11 Apr 2023) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_loops;	// Number of times to send a block of data.
static	long	_qty;	// Number of samples to send per block.



//*****************************************************************************
static void _init_data(const args_t* args, int index)
{
	#define	PIE		(3.1415962)

	int			chans	= 1;	// We only access one channel at a time.
	u32*		dest;
	int			i;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	int			samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");
	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) args->fsamp) * chans;
	samp_tot	= ((float) args->fsamp) * chans * args->seconds;
	samp_dat	= sizeof(_data) / 4;

	period_len	= (args->fsamp <= 100)		? args->fsamp			//      10-      100 ->    10-   100
				: (args->fsamp <= 1000)		? (args->fsamp / 2)		//     101-    1,000 ->    50-   500
				: (args->fsamp <= 10000)	? (args->fsamp / 5)		//   1,001-   10,000 ->   200- 2,000
				: (args->fsamp <= 100000)	? (args->fsamp / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (args->fsamp / 100);								// 100,001-1,000,000 -> 1,000-10,000
	period_len	= (period_len <= 1) ? 2 : period_len;
	period_tot	= period_len * chans;
	period		= (long) (1000000000. / args->fsamp * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_loops	= 1;
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_loops	= args->seconds;
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_loops	= (long) (args->seconds * (samp_sec / period_tot));
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (long) ((samp_dat / chans) * chans);
		_loops	= (long) ((samp_tot + _qty - 1) / _qty);
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		if (index == 0)
			*dest++	= (u16) (0xFFFF - (0xFFFF * samp / (period_len - 1)));

		// 2nd channel: ramp wave
		if (index == 1)
			*dest++	= (u16) (0xFFFF * samp / (period_len - 1));

		// 3rd channel: square wave
		if (index == 2)
			*dest++	= (u16) (((0xFFFF * samp / (period_len - 1)) >= 0x8000) ? 0xFFFF : 0);

		// 4th channel: sine wave
		if (index == 3)
			*dest++	= (u16) (((long) 0x7FFF) * (1 + sin((2 * PIE / period_len) * samp)));
	}

	printf("DONE  (pattern length: %ld samples, %ld %s)\n", period_len, period, units);
}



//*****************************************************************************
static int _io_write(const args_t* args, int errs)
{
	os_time_ns_t	begin;
	os_time_ns_t	end;
	s32				fifo	= 32 * 1024;
	long			loop;
	long long		ms;
	char*			ptr;
	double			rate;
	int				send;
	int				sent;
	long long		total;

	gsc_label("Writing Data");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("SKIPPED  (due to errors)\n");
			break;
		}

		// Prefill the output buffer
		fifo	*= 4;
		total	= 0;

		for (;;)
		{
			sent	= ao4mf_write(args->fd, _data, _qty * 4);

			if (sent < 0)
				errs++;

			if (errs)
				break;

			total	+= sent;

			if (total >= fifo)
				break;
		}

		// Write the data to the device.
		total	= 0;
		os_time_get_ns(&begin);

		for (loop = 0; loop < _loops; loop++)
		{
			if (errs)
				break;

			send	= _qty * 4;
			ptr		= (char*) _data;

			for (;;)
			{
				sent	= ao4mf_write(args->fd, ptr, send);

				if (sent < 0)
					errs++;

				if (sent <= 0)
					break;

				total	+= sent;
				ptr		+= sent;
				send	-= sent;

				if (send == 0)
					break;
			}

			if (sent <= 0)
				break;
		}

		// Compute the results.
		os_time_get_ns(&end);

		if (errs)
		{
			errs	= 1;
			printf("FAIL <---  (write error, ");
		}
		else if (sent == 0)
		{
			errs	= 1;
			printf("FAIL <---  (I/O timeout, ");
		}
		else
		{
			printf("PASS  (");
		}

		// Samples
		gsc_label_long_comma(total / 4);
		printf(" Samples, ");

		// Period
		ms	= ((long long) end.tv_sec * 1000 + end.tv_nsec / 1000000)
			- ((long long) begin.tv_sec * 1000 + begin.tv_nsec / 1000000);
		gsc_label_float_comma((double) ms / 1000, -1, 3);
		printf(" Seconds, ");

		// Rate
		ms		= (ms <= 0) ? 1 : ms;
		rate	= ((double) total * 1000000 / 4) / ms / 1000;
		gsc_label_float_comma(rate, -1, 3);
		printf(" S/S)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	s32	index;
	s32	rate;

	errs	+= ao4mf_config_ao(args->fd, -1, 1, args->fsamp);
	errs	+= ao4mf_tx_io_mode(args->fd, -1, 1, args->io_mode, NULL);

	gsc_label("Analog Output");
	errs	+= ao4mf_query(args->fd, -1, 0, AO4MF_QUERY_INDEX_CHANNEL, &index);
	errs	+= ao4mf_fsamp_report(args->fd, -1, 0, index, &rate);

	if (errs == 0)
	{
		printf("\n");
		gsc_label_level_inc();

		_init_data(args, index);
		errs	+= ao4mf_ao_buf_clear(args->fd, -1, 1);
		errs	+= _io_write(args, errs);

		gsc_label_level_dec();
	}

	return(errs);
}


