// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/din/perform.c $
// $Rev: 52824 $
// $Date: 2023-04-11 12:23:25 -0500 (Tue, 11 Apr 2023) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	DIO_BITS	8

#ifdef CLOCK_MONOTONIC_RAW
#define	CLOCK		CLOCK_MONOTONIC_RAW
#else
#ifdef CLOCK_MONOTONIC
#define	CLOCK		CLOCK_MONOTONIC
#else
#define	CLOCK		CLOCK_REALTIME
#endif
#endif



//*****************************************************************************
#ifndef	CLOCK_MONOTONIC_RAW
#define	clock_gettime(c,ts)	_my_clock_gettime(ts)

static int _my_clock_gettime(struct timespec* ts)
{
	int				ret;
	struct timeval	tv;

	if (ts)
	{
		ret	= 0;
		gettimeofday(&tv, NULL);
		ts->tv_sec	= tv.tv_sec;
		ts->tv_nsec	= tv.tv_usec * 1000;
	}
	else
	{
		ret	= 1;
	}

	return(ret);
}
#endif



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ao4mf_initialize(args->fd, -1, 1);
	errs	+= ao4mf_dio_dir(args->fd, -1, 1, AO4MF_DIO_DIR_IN, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("  Seq   DIO In                                   \n");
	printf("  No.   7654-3210  Time Stamp      Delta         \n");
	printf("  ====  =========  ==============  ==============\n");
}



//*****************************************************************************
static int _monitor(const args_t* args)
{
	u32				bit;
	int				count	= 0;
	long long		delta;
	int				errs	= 0;
	int				i;
	s32				last	= -1;
	time_t			now;
	s32				port;
	time_t			t_limit;
	struct timespec	t0;
	struct timespec	ts;
	struct timespec	ts_last;
	u32				value;

	gsc_label("Digital Input");
	printf("\n");
	gsc_label_level_inc();

	clock_gettime(CLOCK, &t0);
	clock_gettime(CLOCK, &ts);

	_output_header();
	t_limit	= time(NULL) + args->seconds + 1;

	for (; errs == 0;)
	{
		errs	= ao4mf_dio_rx(args->fd, -1, 0, &port);
		value	= port;

		if ((count <= 0) || (value != last))
		{
			ts_last	= ts;
			clock_gettime(CLOCK, &ts);

			if (count <= 0)
			{
				t0		= ts;
				ts_last	= ts;
			}

			count++;
			printf("  %-4d  ", count);

			for (i = DIO_BITS - 1; i >= 0; i--)
			{
				bit	= 0x1 << i;
				printf("%s", (value & bit) ? "1" : "0");

				if (bit & 0x0010)
					printf(" ");
			}

			delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
					- (1000000000LL * t0.tv_sec + t0.tv_nsec);
			printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

			delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
					- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
			printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

			printf("\n");
			fflush(stdout);
			last	= value;
		}

		now	= time(NULL);

		if (now >= t_limit)
			break;
	}

	printf("\n");
	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	diopr	= 0;
	int	errs	= 0;

	errs	+= ao4mf_query(args->fd, -1, 1, AO4MF_QUERY_REG_DIOPCR, &diopr);

	if (diopr == 0)
	{
		gsc_label("DIO Testing");
		printf("SKIPPING  (DIO not supported on this channel.)\n");
	}
	else
	{
		errs	+= _config(args);
		errs	+= _monitor(args);
	}

	return(errs);
}


