// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_ao_buf_status.c $
// $Rev: 43274 $
// $Date: 2018-07-09 17:36:40 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao4mf_ao_buf_status
*
*	Purpose:
*
*		Provide a visual wrapper for the AO4MF_IOCTL_AO_BUF_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao4mf_ao_buf_status(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("AO Buffer Status", index);

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_AO_BUF_STATUS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AO4MF_AO_BUF_STATUS_EMPTY:

			ptr	= "Empty";
			break;

		case AO4MF_AO_BUF_STATUS_1Q_FULL:

			ptr	= "< 1/4 Full";
			break;

		case AO4MF_AO_BUF_STATUS_MEDIUM:

			ptr	= "Medium (>= 1/4, <= 3/4)";
			break;

		case AO4MF_AO_BUF_STATUS_3Q_FULL:

			ptr	= "> 3/4 Full";
			break;

		case AO4MF_AO_BUF_STATUS_FULL:

			ptr	= "Full";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


