// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_config_ao.c $
// $Rev: 52822 $
// $Date: 2023-04-11 12:21:11 -0500 (Tue, 11 Apr 2023) $

// 16AO4MF: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao4mf_config_ao
*
*	Purpose:
*
*		Configure the basic services.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ao4mf_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	s32	chan;
	int	errs	= 0;
	s32	irate;
	s32	nclk	= 0;
	s32	nrate	= 0;
	s32	src		= 0;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_INDEX_CHANNEL, &chan);

	errs	+= ao4mf_initialize			(fd, index, verbose);
	errs	+= ao4mf_auto_cal			(fd, index, verbose);
	errs	+= ao4mf_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,			NULL);
	errs	+= ao4mf_tx_io_timeout		(fd, index, verbose, AO4MF_IO_TIMEOUT_DEFAULT,	NULL);

	errs	+= ao4mf_ao_buf_mode		(fd, index, verbose, AO4MF_AO_BUF_MODE_OPEN,	NULL);
	errs	+= ao4mf_ao_buf_size		(fd, index, verbose, AO4MF_AO_BUF_SIZE_32K,		NULL);
	errs	+= ao4mf_burst_mode			(fd, index, verbose, AO4MF_BURST_MODE_DISABLE,	NULL);
	errs	+= ao4mf_xcvr_in			(fd, index, verbose, AO4MF_XCVR_IN_TTL,			NULL);
	errs	+= ao4mf_data_format		(fd, index, verbose, AO4MF_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= ao4mf_ao_clk_src			(fd, index, verbose, AO4MF_AO_CLK_SRC_INT,		NULL);

	if (chan == 0)
		errs	+= ao4mf_rate_gen_sel	(fd, index, verbose, AO4MF_RATE_GEN_SEL_CH_0,	NULL);
	else
		errs	+= ao4mf_rate_gen_sel	(fd, index, verbose, AO4MF_RATE_GEN_SEL_CH_ALT,	NULL);

	errs	+= ao4mf_fsamp_compute		(fd, index, verbose, fsamp, &irate, &nrate, &src, &nclk, NULL);
	errs	+= ao4mf_int_samp_rate		(fd, index, verbose, irate,						NULL);
	errs	+= ao4mf_nrate				(fd, index, verbose, nrate,						NULL);
	errs	+= ao4mf_nclk				(fd, index, verbose, nclk,						NULL);
	errs	+= ao4mf_fref_clk_src		(fd, index, verbose, src,						NULL);
	errs	+= ao4mf_ao_clk_enable		(fd, index, verbose, AO4MF_AO_CLK_ENABLE_YES,	NULL);
	errs	+= ao4mf_fsamp_report_all	(fd, index, verbose,							NULL);

	errs	+= ao4mf_ao_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


