// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_dio_dir.c $
// $Rev: 50603 $
// $Date: 2022-02-28 08:16:41 -0600 (Mon, 28 Feb 2022) $

// 16AO4MF: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao4mf_dio_dir
*
*	Purpose:
*
*		Provide a visual wrapper for the AO4MF_IOCTL_DIO_DIR service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao4mf_dio_dir(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Digital I/O Direction", index);

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_DIO_DIR, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "Feature Not Unsupported";
			break;

		case AO4MF_DIO_DIR_IN:

			ptr	= "Input";
			break;

		case AO4MF_DIO_DIR_OUT:

			ptr	= "Output";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


