// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_fsamp_ao.c $
// $Rev: 52822 $
// $Date: 2023-04-11 12:21:11 -0500 (Tue, 11 Apr 2023) $

// 16AO4MF: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))



// data types *****************************************************************

typedef struct
{
	s32			fsamp_want;		// value provided
	s32			fref;			// value provided/updated

	s32			nclk_max;		// queried value
	s32			nclk_min;		// queried value
	s32			nrate_max;		// queried value
	s32			nrate_min;		// queried value

	s32			fref_src;		// value computed
	double		delta;			// value updated
	double		fsamp_got;		// value computed
	s32			irate;			// value computed
	s32			nclk;			// value computed
	s32			nrate;			// value computed
} fsamp_t;



//*****************************************************************************
static void _fsamp_adj_compute(fsamp_t* data)
{
	double	adj;
	double	delta;
	double	fsamp_got;
	s32		nclk;
	s32		nclk_ll;
	s32		nclk_ul;
	s32		nrate;

	data->fref_src	= AO4MF_FREF_CLK_SRC_ADJ;
	data->irate		= AO4MF_INT_SAMP_RATE_VAR;

	for (nrate = data->nrate_min; nrate <= data->nrate_max; nrate++)
	{
		adj		= (double) data->fsamp_want * nrate;
		nclk	= (s32) ((511.0 / data->fref) * (adj - data->fref));
		nclk_ll	= nclk - 1;
		nclk_ul	= nclk + 1;
		nclk_ll	= (nclk_ll < data->nclk_min) ? data->nclk_min : nclk_ll;
		nclk_ul	= (nclk_ul > data->nclk_max) ? data->nclk_max : nclk_ul;

		for (nclk = nclk_ll; nclk <= nclk_ul; nclk++)
		{
			adj			= (double) data->fref * (1 + (double) nclk / 511);
			fsamp_got	= adj / nrate;
			delta		= fsamp_got - data->fsamp_want;

			if ((ABS(delta)) < (ABS(data->delta)))
			{
				data->delta		= delta;
				data->nrate		= nrate;
				data->nclk		= nclk;
				data->fsamp_got	= fsamp_got;
			}
		}
	}
}



//*****************************************************************************
static void _fsamp_fixed_compute(fsamp_t* data)
{
	static const struct
	{
		int	rate;
		s32	irate;
	} list[]	=
	{
		{ 10000,	AO4MF_INT_SAMP_RATE_10000_SPS	},
		{ 22500,	AO4MF_INT_SAMP_RATE_22500_SPS	},
		{ 27304,	AO4MF_INT_SAMP_RATE_27304_SPS	},
		{ 60000,	AO4MF_INT_SAMP_RATE_60000_SPS	},
		{ 224000,	AO4MF_INT_SAMP_RATE_224000_SPS	},
		{ 0, 0 }
	};

	double	delta;
	double	fsamp_got;
	int		i;

	data->fref_src	= -1;

	for (i = 0; list[i].rate; i++)
	{
		fsamp_got	= (double) list[i].rate;
		delta		= fsamp_got - data->fsamp_want;

		if ((ABS(delta)) < (ABS(data->delta)))
		{
			data->delta		= delta;
			data->irate		= list[i].irate;
			data->fsamp_got	= fsamp_got;
		}
	}
}



//*****************************************************************************
static void _fsamp_master_compute(fsamp_t* data)
{
	double	delta;
	double	fsamp_got;
	s32		nrate;
	s32		nrate_ll;
	s32		nrate_ul;

	data->fref_src	= AO4MF_FREF_CLK_SRC_MASTER;
	data->irate		= AO4MF_INT_SAMP_RATE_VAR;

	nrate			= (s32) ((double) data->fref / data->fsamp_want);
	nrate_ll		= nrate - 1;
	nrate_ul		= nrate + 1;
	nrate_ll		= (nrate_ll < data->nrate_min) ? data->nrate_min : nrate_ll;
	nrate_ul		= (nrate_ul > data->nrate_max) ? data->nrate_max : nrate_ul;

	for (nrate = nrate_ll; nrate <= nrate_ul; nrate++)
	{
		fsamp_got	= ((double) data->fref) / nrate;
		delta		= fsamp_got - data->fsamp_want;

		if ((ABS(delta)) < (ABS(data->delta)))
		{
			data->delta		= delta;
			data->fsamp_got	= fsamp_got;
			data->nrate		= nrate;
		}
	}
}



//*****************************************************************************
int	ao4mf_fsamp_compute(int fd, int index, int verbose, s32 fsamp, s32* irate, s32* nrate, s32* fref_src, s32* nclk, double* rate)
{
	fsamp_t	data;
	fsamp_t	data_adj;
	fsamp_t	data_fixed;
	fsamp_t	data_master;
	int		errs		= 0;

	if (verbose)
	{
		gsc_label_index("AO Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	memset(&data, 0, sizeof(data));
	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_NRATE_MAX, &data.nrate_max);
	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_NRATE_MIN, &data.nrate_min);
	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_NCLK_MAX, &data.nclk_max);
	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_NCLK_MIN, &data.nclk_min);

	data.fsamp_want	= fsamp;
	data.fref_src	= -1;
	data.delta		= FLT_MAX;
	data.irate		= -1;
	data.nclk		= -1;
	data.nrate		= -1;

	data_adj		= data;
	data_fixed		= data;
	data_master		= data;

	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_FREF_ADJ, &data_adj.fref);
	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_FREF_MASTER, &data_master.fref);

	_fsamp_adj_compute(&data_adj);
	_fsamp_fixed_compute(&data_fixed);
	_fsamp_master_compute(&data_master);

	if ((ABS(data_adj.delta)) < (ABS(data_fixed.delta)))
		data	= data_adj;
	else
		data	= data_fixed;

	if ((ABS(data_master.delta)) < (ABS(data.delta)))
		data	= data_master;

	if (irate)
		irate[0]	= data.irate;

	if (nrate)
		nrate[0]	= data.nrate;

	if (fref_src)
		fref_src[0]	= data.fref_src;

	if (nclk)
		nclk[0]		= data.nclk;

	if (rate)
		rate[0]	= data.fsamp_got;

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");

		switch (data.fref_src)
		{
			default:
			case -1:	// fixed

				printf("Fixed, ");
				break;

			case AO4MF_FREF_CLK_SRC_ADJ:

				printf("Adjustable");
				printf(", Fref ");
				gsc_label_long_comma(data.fref);
				printf(", Nrate ");
				gsc_label_long_comma(data.nrate);
				printf(", Nclk ");
				gsc_label_long_comma(data.nclk);
				printf(", ");
				break;

			case AO4MF_FREF_CLK_SRC_MASTER:

				printf("Master");
				printf(", Fref ");
				gsc_label_long_comma(data.fref);
				printf(", Nrate ");
				gsc_label_long_comma(data.nrate);
				printf(", ");
				break;
		}

		printf("Fsamp %.3f", data.fsamp_got);
		printf(")\n");
	}

	return(errs);
}



//*****************************************************************************
int ao4mf_fsamp_report(int fd, int index, int verbose, s32 chan, s32* sps)
{
	s32		a_fref;
	char	buf[64];
	s32		burst;
	s32		clk;
	int		errs	= 0;
	s32		fref_src;
	s32		fsamp	= 0;
	s32		irate;
	s32		m_fref;
	s32		nclk;
	s32		nrate;
	s32		rgen;
	s32		src;

	if (chan < 0)
		errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_INDEX_CHANNEL, &chan);

	if (verbose)
	{
		sprintf(buf, "Fsamp Ch %d", chan);
		gsc_label_index(buf, index);
	}

	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_FREF_ADJ, &a_fref);
	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_FREF_MASTER, &m_fref);
	errs	+= ao4mf_fref_clk_src	(fd, -1, 0, -1, &fref_src);
	errs	+= ao4mf_nrate			(fd, -1, 0, -1, &nrate);
	errs	+= ao4mf_nclk			(fd, -1, 0, -1, &nclk);
	errs	+= ao4mf_burst_mode		(fd, -1, 0, -1, &burst);
	errs	+= ao4mf_ao_clk_enable	(fd, -1, 0, -1, &clk);
	errs	+= ao4mf_ao_clk_src		(fd, -1, 0, -1, &src);
	errs	+= ao4mf_int_samp_rate	(fd, -1, 0, -1, &irate);
	errs	+= ao4mf_rate_gen_sel	(fd, -1, 0, -1, &rgen);

	if (errs)
	{
	}
	else if (burst == AO4MF_BURST_MODE_ENABLE)
	{
		if (verbose)
			printf("UNKNOWN  (Bursting is enabled.)\n");
	}
	else if (clk == AO4MF_AO_CLK_ENABLE_NO)
	{
		if (verbose)
			printf("UNKNOWN  (Clocking is disabled.)\n");
	}
	else if (src == AO4MF_AO_CLK_SRC_EXT_SW)
	{
		if (verbose)
			printf("UNKNOWN  (Not using internal clock.)\n");
	}
	else if (((chan == 0) && (rgen != AO4MF_RATE_GEN_SEL_CH_0)) ||
			 ((chan > 0) && (rgen != AO4MF_RATE_GEN_SEL_CH_ALT)))
	{
		if (verbose)
			printf("UNKNOWN  (Not using this channel's rate generator.)\n");
	}
	else if (irate != AO4MF_INT_SAMP_RATE_VAR)
	{
		switch (irate)
		{
			default:								fsamp	= -1;		break;
			case AO4MF_INT_SAMP_RATE_10000_SPS:		fsamp	= 10000;	break;
			case AO4MF_INT_SAMP_RATE_22500_SPS:		fsamp	= 22500;	break;
			case AO4MF_INT_SAMP_RATE_27304_SPS:		fsamp	= 27304;	break;
			case AO4MF_INT_SAMP_RATE_60000_SPS:		fsamp	= 60000;	break;
			case AO4MF_INT_SAMP_RATE_224000_SPS:	fsamp	= 224000;	break;
		}

		if (fsamp < 0)
		{
			errs++;
			printf(	"ERROR <---  (Invalid fixed rate option: %ld)\n",
					(long) irate);
		}
		else
		{
			if (verbose)
			{
				gsc_label_long_comma(fsamp);
				printf(" S/S (Fixed Rate)\n");
			}
		}
	}
	else if (fref_src == AO4MF_FREF_CLK_SRC_MASTER)
	{
		fsamp	= (s32) ((double) m_fref / nrate + 0.5);

		if (verbose)
		{
			gsc_label_long_comma(fsamp);
			printf(" S/S  (Master, Fref ");
			gsc_label_long_comma(m_fref);
			printf(", Nrate %ld)\n", (long) nrate);
		}
	}
	else if (fref_src == AO4MF_FREF_CLK_SRC_ADJ)
	{
		fsamp	= (s32) ((double) a_fref * (1 + (double) nclk / 511) / nrate + 0.5);

		if (verbose)
		{
			gsc_label_long_comma(fsamp);
			printf(" S/S  (Adjustable, Fref ");
			gsc_label_long_comma(a_fref);
			printf(", Nrate %ld", (long) nrate);
			printf(", Nclk %ld)\n", (long) nclk);
		}
	}
	else
	{
		errs++;

		if (verbose)
		{
			gsc_label_long_comma(fsamp);
			printf(" S/S  FAIL <--- (Internal error: invalid Fref Source)\n");
		}
	}

	sps[0]	= fsamp;
	return(errs);
}



//*****************************************************************************
int ao4mf_fsamp_report_all(int fd, int index, int verbose, s32* fsamp)
{
	s32	chan;
	int	errs	= 0;
	s32	sps;

	if (verbose)
	{
		gsc_label_index("Sample Rates", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ao4mf_query(fd, -1, 0, AO4MF_QUERY_INDEX_CHANNEL, &chan);
	errs	+= ao4mf_fsamp_report(fd, index, verbose, chan, &sps);

	if (verbose)
	{
		gsc_label("Overall Rate");
		gsc_label_long_comma(sps);
		printf(" S/S\n");
		gsc_label_level_dec();
	}

	if (fsamp)
		fsamp[0]	= sps;

	return(errs);
}


