// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_id.c $
// $Rev: 52822 $
// $Date: 2023-04-11 12:21:11 -0500 (Tue, 11 Apr 2023) $

// 16AO4MF: Utilities: source file

#include "main.h"



//*****************************************************************************
static int _id_device_pci(int fd)
{
	int	errs;
	u32	reg;

	gsc_label("Vendor ID");
	errs	= ao4mf_reg_read(fd, -1, 0, GSC_PCI_9080_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= ao4mf_reg_read(fd, -1, 0, GSC_PCI_9080_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9080)
	{
		printf("(PCI9080)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= ao4mf_reg_read(fd, -1, 0, GSC_PCI_9080_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= ao4mf_reg_read(fd, -1, 0, GSC_PCI_9080_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x3064)
	{
		printf("(16AO4MF/S)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



//*****************************************************************************
static int _id_device_frr(int fd)
{
	int	errs;
	u32	reg;

	gsc_label("Firmware Rev Reg");
	errs	= ao4mf_reg_read(fd, -1, 0, AO4MF_GSC_FRR, &reg);
	printf("0x%08lX\n", (long) reg);

	return(errs);
}



/******************************************************************************
*
*	Function:	ao4mf_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ao4mf_id_device(int fd, int index, int verbose)
{
	char	buf[80]		= "";
	s32		device_type;
	int		errs;

	if (verbose)
		gsc_label_index("Device", index);

	errs	= ao4mf_query(fd, -1, 0, AO4MF_QUERY_DEVICE_TYPE, &device_type);

	switch (device_type)
	{
		default:

			errs	= 1;
			sprintf(buf,
					"FAIL <---  (unexpected device type: %ld)",
					(long) device_type);
			break;

		case GSC_DEV_TYPE_16AO4MF:

			strcpy(buf, "16AO4MF/S");
			break;
	}

	if (verbose)
	{
		printf("%s\n", buf);
		gsc_label_level_inc();

		errs	+= _id_device_pci(fd);
		errs	+= _id_device_frr(fd);

		gsc_label_level_dec();
	}

	return(errs);
}



