// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_int_samp_rate.c $
// $Rev: 43274 $
// $Date: 2018-07-09 17:36:40 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao4mf_int_samp_rate
*
*	Purpose:
*
*		Provide a visual wrapper for the AO4MF_IOCTL_INT_SAMP_RATE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao4mf_int_samp_rate(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Internal Sample Rate", index);

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_INT_SAMP_RATE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AO4MF_INT_SAMP_RATE_VAR:

			ptr	= "Variable";
			break;

		case AO4MF_INT_SAMP_RATE_224000_SPS:

			ptr	= "224,000 S/S";
			break;

		case AO4MF_INT_SAMP_RATE_60000_SPS:

			ptr	= "60,000 S/S";
			break;

		case AO4MF_INT_SAMP_RATE_27304_SPS:

			ptr	= "27,304 S/S";
			break;

		case AO4MF_INT_SAMP_RATE_22500_SPS:

			ptr	= "22,500 S/S";
			break;

		case AO4MF_INT_SAMP_RATE_10000_SPS:

			ptr	= "10,000 S/S";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


