// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_irq_sel.c $
// $Rev: 43274 $
// $Date: 2018-07-09 17:36:40 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao4mf_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AO4MF_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao4mf_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("AO4MF_IOCTL_IRQ_SEL", index);

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AO4MF_IRQ_SEL_NONE_INIT_DONE:

			ptr	= "None, Initialize Done";
			break;

		case AO4MF_IRQ_SEL_AUTO_CAL_DONE:

			ptr	= "Auto-Cal Done";
			break;

		case AO4MF_IRQ_SEL_AO_BUF_EMPTY:

			ptr	= "AO Buffer Empty";
			break;

		case AO4MF_IRQ_SEL_AO_BUF_1Q_FULL:

			ptr	= "AO Buffer 1/4 Full";
			break;

		case AO4MF_IRQ_SEL_AO_BUF_3Q_FULL:

			ptr	= "AO Buffer 3/4 Full";
			break;

		case AO4MF_IRQ_SEL_BURST_TRIG_READY:

			ptr	= "Burst Trigger Ready";
			break;

		case AO4MF_IRQ_SEL_LOAD_READY:

			ptr	= "Load Ready";
			break;

		case AO4MF_IRQ_SEL_LOAD_READY_END:

			ptr	= "Load Ready End";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


