// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_reg.c $
// $Rev: 50598 $
// $Date: 2022-02-24 19:56:18 -0600 (Thu, 24 Feb 2022) $

// 16AO4MF: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)		(sizeof((a))/sizeof((a)[0]))

#define	_GSC_REG(a)			"GSC " #a, AO4MF_GSC_##a, 0, 0
#define	WIDTH				25

#define	FIELD_SHOW(beg,end,eol,lst,dsc)		\
			gsc_reg_field_show(width+10,WIDTH,value,(beg),(end),(eol),(lst),(dsc))

#define	QUERY_INDEX_CHANNEL(fd, ptr)	ao4mf_query((fd), -1, 0, AO4MF_QUERY_INDEX_CHANNEL, (ptr))



//*****************************************************************************
static int _gsc_bcr(int fd, int supported, u32 value, int width)
{
	static const char*	_irq[]	=
	{
		"None, Initialize Done",
		"Auto-Cal Done",
		"Output Buffer Empty",
		"Output Buffer < 1/4 Full",
		"Output Buffer > 3/4 Full",
		"Burst Trigger Ready",
		"Load Ready",
		"End Load Ready"
	};

	static const char*	_cal[]	=
	{
		"Idle",
		"RESERVED",
		"Active",
		"RESERVED"
	};

	static const char*	_0_1[]	= { "Channel 0",			"Channel 1"	};
	static const char*	_1_0[]	= { "Channel 1",			"Channel 0"	};
	static const char*	_2_0[]	= { "Channel 2",			"Channel 0"	};
	static const char*	_3_0[]	= { "Channel 3",			"Channel 0"	};
	static const char*	_idle[]	= { "Idle",					"Active"	};
	static const char*	_lvds[]	= { "No, LVDS",				"Yes"		};
	static const char*	_no[]	= { "No",					"Yes"		};
	static const char*	_pass[]	= { "Pass",					"Yes"		};
	static const char*	_twos[]	= { "No, Two's Compliment",	"Yes"		};

	int	errs;
	s32	index	= 0;

	errs	= QUERY_INDEX_CHANNEL(fd, &index);

	FIELD_SHOW( 0,  0, 1, _no,		"Burst Enabled"				);
	FIELD_SHOW( 1,  1, 1, _no,		"Burst Ready"				);
	FIELD_SHOW( 2,  2, 1, _idle,	"Burst Trigger"				);

	if (index == 0)
		FIELD_SHOW( 3,  3, 1, _pass,	"Global Auto-Cal Status"	);
	else
		FIELD_SHOW( 3,  3, 1, NULL,		"Reserved"				);

	FIELD_SHOW( 4,  4, 1, _twos,	"Offset Binary"				);
	FIELD_SHOW( 5,  5, 1, _lvds,	"TTL SYNC I/O"				);

	if (index == 0)
		FIELD_SHOW( 6,  6, 1, _idle,	"Global Auto-Cal"		);
	else
		FIELD_SHOW( 6,  6, 1, NULL,		"Reserved"				);


	switch (index)
	{
		default:

			errs++;
			printf(	"FAIL <--- (Unrecognixed channel index: %ld)\n",
					(long) index);
			break;

		case 0:	FIELD_SHOW( 7,  7, 1, _0_1,	"Common Rate Generator"	);	break;
		case 1:	FIELD_SHOW( 7,  7, 1, _1_0,	"Common Rate Generator"	);	break;
		case 2:	FIELD_SHOW( 7,  7, 1, _2_0,	"Common Rate Generator"	);	break;
		case 3:	FIELD_SHOW( 7,  7, 1, _3_0,	"Common Rate Generator"	);	break;
	}

	FIELD_SHOW(10,  8, 1, _irq,		"Interrupt"			);
	FIELD_SHOW(11, 11, 1, _idle,	"Interrupt Flag"	);
	FIELD_SHOW(13, 12, 1, _cal,		"Calibration Mode"	);
	FIELD_SHOW(14, 14, 1, _pass,	"Auto-Cal Status"	);
	FIELD_SHOW(15, 15, 1, _idle,	"Initialize"		);
	FIELD_SHOW(31, 16, 1, NULL,		"Reserved"			);

	return(errs);
}



//*****************************************************************************
static int _gsc_diopcr(int fd, int supported, u32 value, int width)
{
	static const char*	_in[]	= { "No, Input",	"Yes"	};

	s32	index	= 0;
	int	ret;

	if (supported)
	{
		QUERY_INDEX_CHANNEL(fd, &index);
		ret	= (index == 0) ? 1 : 0;
	}
	else
	{
		FIELD_SHOW( 7, 0, 1, NULL,	"Digital I/O"		);
		FIELD_SHOW( 8, 8, 1, _in,	"Digital Output"	);
		FIELD_SHOW(31, 9, 1, NULL,	"Reserved"			);
		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
static int _gsc_srcr(int fd, int supported, u32 value, int width)
{
	FIELD_SHOW(15,  0, 1, NULL,	"Nrate"		);
	FIELD_SHOW(31, 16, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _gsc_bor(int fd, int supported, u32 value, int width)
{
	static const char*	_size[]	=
	{
		"4",
		"8",
		"16",
		"32",
		"64",
		"128",
		"256",
		"512",
		"1K",
		"2K",
		"4K",
		"8K",
		"16K",
		"32K",
		"RESERVED",
		"RESERVED"
	};

	static const char*	_disable[]	= { "Disable",		"Enable"	};
	static const char*	_idle[]		= { "Idle",			"Active"	};
	static const char*	_int[]		= { "No, Internal",	"Yes"		};
	static const char*	_no[]		= { "No",			"Yes"		};
	static const char*	_open[]		= { "No, Open",		"Yes"		};

	FIELD_SHOW( 3,  0, 1, _size,	"Buffer Size"			);
	FIELD_SHOW( 4,  4, 1, _int,		"External Clock"		);
	FIELD_SHOW( 5,  5, 1, _disable,	"Enable Clock"			);
	FIELD_SHOW( 6,  6, 1, _no,		"Clock Ready"			);
	FIELD_SHOW( 7,  7, 1, _idle,	"SW Clock"				);
	FIELD_SHOW( 8,  8, 1, _open,	"Circular Buffer"		);
	FIELD_SHOW( 9,  9, 1, _idle,	"Load Request"			);
	FIELD_SHOW(10, 10, 1, _no,		"Load Ready"			);
	FIELD_SHOW(11, 11, 1, _idle,	"Clear Buffer"			);
	FIELD_SHOW(12, 12, 1, _no,		"Buffer Empty"			);
	FIELD_SHOW(13, 13, 1, _no,		"Buffer Low Quarter"	);
	FIELD_SHOW(14, 14, 1, _no,		"Buffer High Quarter"	);
	FIELD_SHOW(15, 15, 1, _no,		"Buffer Full"			);
	FIELD_SHOW(31, 16, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _gsc_frr(int fd, int supported, u32 value, int width)
{
	FIELD_SHOW(15,  0, 1, NULL,	"Firmware Revision"	);
	FIELD_SHOW(31, 16, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _gsc_accr(int fd, int supported, u32 value, int width)
{
	static const char*	_master[]	= { "No, Use Master",	"Yes"	};

	FIELD_SHOW( 8,  0, 1, NULL,		"Nclk"						);
	FIELD_SHOW( 9,  9, 1, _master,	"Select Adjustable Clock"	);
	FIELD_SHOW(31, 10, 1, NULL,		"Reserved"					);
	return(0);
}



//*****************************************************************************
static int _gsc_icscr(int fd, int supported, u32 value, int width)
{
	static const char*	_src[]	=
	{
		"Variable Clock",
		"224,000.00 S/S",
		"60,000.00 S/S",
		"27,304.00 S/S",
		"22,500.00 S/S",
		"10,000.00 S/S",
		"10,000.00 S/S",
		"10,000.00 S/S",
		"10,000.00 S/S",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};


	FIELD_SHOW( 3, 0, 1, _src,	"Internal Clock Source"	);
	FIELD_SHOW(31, 4, 1, NULL,	"Reserved"				);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCR),	0, _gsc_bcr,	"Board Control Register"			},
	{ _GSC_REG(DIOPCR),	1, _gsc_diopcr,	"Digital I/O Port Control Register"	},
	{ _GSC_REG(SRCR),	0, _gsc_srcr,	"Sample Rate Control Register"		},
	{ _GSC_REG(BOR),	0, _gsc_bor,	"Buffer Operations Register"		},
	{ _GSC_REG(FRR),	0, _gsc_frr,	"Firmware Revision Register"		},
	{ _GSC_REG(ADR),	0, NULL,		"Autocal Diagnostics Register"		},
	{ _GSC_REG(ODBR),	0, NULL,		"Output Data Buffer Register"		},
	{ _GSC_REG(ACCR),	0, _gsc_accr,	"Adjustable Clock Control Register"	},
	{ _GSC_REG(ICSCR),	0, _gsc_icscr,	"Internal Clock Src Ctrl Register"	},
	{ NULL }
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	ao4mf_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ao4mf_reg_get_def_id(int reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	ao4mf_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ao4mf_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	for (;;)	// A convenience loop.
	{
		if (index < 0)
		{
			def	= NULL;
			break;
		}

		if (index < ((int) SIZEOF_ARRAY(_gsc) - 1))
		{
			def	= &_gsc[index];
			break;
		}

		def	= NULL;
		break;
	}

	return(def);
}



/******************************************************************************
*
*	Function:	ao4mf_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ao4mf_reg_get_desc(int reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	ao4mf_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ao4mf_reg_get_name(int reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	ao4mf_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ao4mf_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, ao4mf_reg_read);
	return(errs);
}


